<?php

declare(strict_types = 1);

namespace Drupal\group_permissions_parameter\Form;

use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\group_permissions_parameter\GroupPermissionsSyncBatch;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Sync group permissions form.
 */
class SyncGroupPermissionsForm extends FormBase {

  /**
   * The group permissions parameter sync batch service.
   *
   * @var \Drupal\group_permissions_parameter\GroupPermissionsSyncBatch
   */
  protected $groupPermissionsParameterSyncBatch;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  public $entityTypeBundleInfo;

  /**
   * Constructs a new SyncGroupPermissionsForm object.
   *
   * @param \Drupal\group_permissions_parameter\GroupPermissionsSyncBatch $group_permissions_parameter_sync_batch
   *   The group permissions parameter sync batch service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   */
  public function __construct(GroupPermissionsSyncBatch $group_permissions_parameter_sync_batch, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info) {
    $this->groupPermissionsParameterSyncBatch = $group_permissions_parameter_sync_batch;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('group_permissions_parameter.sync_batch'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'group_permissions_parameter_sync_permissions_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['group_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Group types'),
      '#description' => $this->t('Select the group types that you would like to sync the group permissions for.'),
      '#options' => $this->getGroupTypes(),
      '#required' => TRUE,
      '#multiple' => TRUE,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Sync permissions'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->groupPermissionsParameterSyncBatch->build($form_state->getValue('group_types'));
  }

  /**
   * Get all group types.
   *
   * @return array
   *   The group types.
   */
  protected function getGroupTypes(): array {
    $entity_type = $this->entityTypeManager->getDefinition('group');
    $bundles = $this->entityTypeBundleInfo->getBundleInfo('group');

    // @todo exclude the group types that doesn't support group permissions when
    //   https://www.drupal.org/project/group_permissions/issues/3276738 is
    //   fixed.
    if ($entity_type->hasKey('bundle')) {
      $bundle_options = [];
      foreach ($bundles as $bundle_name => $bundle_info) {
        // Get the count of the groups by the bundle.
        $group_storage = $this->entityTypeManager->getStorage('group');
        $group_ids = $group_storage->getQuery()
          ->condition('type', $bundle_name)
          ->execute();
        $bundle_options[$bundle_name] = $bundle_info['label'] . ' (' . count($group_ids) . ')';
      }

      return $bundle_options;
    }

    return [];
  }

}
